#include <Servo.h>
#define RIGHT_MOTOR_PIN 5
#define LEFT_MOTOR_PIN 3
#define ARM_MOTOR_PIN 6
#define LEFT_IR_PIN 4
#define RIGHT_IR_PIN 5
#define MAGNET_PIN 13
#define RIGHT_MOTOR_STOP 1504
#define LEFT_MOTOR_STOP 1504
#define SPEED_CORRECTION 0.93 // Decrease if go left; increase if go right
#define ARM_UP_POS 0
#define ARM_DOWN_POS 130
#define TRIGGER_IR_DST 300

// Times (ms)
#define DELTA_RIDE_TIME 100
#define DELTA_CROSS_TIME 3580
#define DELTA_ROTATION_TIME 2800
#define ARM_MOVING_TIME 1400
#define DELTA_ALIGN_TIME 50

int state=0;
Servo rightMotor, leftMotor, armMotor;

void goForwardTillGap(int velocity)
{
	while (!detectGap())
	{
		rightMotor.writeMicroseconds(RIGHT_MOTOR_STOP-SPEED_CORRECTION*velocity);
		leftMotor.writeMicroseconds(LEFT_MOTOR_STOP+velocity);
		delay(DELTA_RIDE_TIME);
	}
	rightMotor.writeMicroseconds(RIGHT_MOTOR_STOP);
	leftMotor.writeMicroseconds(LEFT_MOTOR_STOP);
	align(velocity);
	state=2;
}

void putBridge()
{
	digitalWrite(MAGNET_PIN, HIGH);
	armMotor.write(ARM_DOWN_POS);
	delay(ARM_MOVING_TIME);
	digitalWrite(MAGNET_PIN, LOW);
	armMotor.write(ARM_UP_POS);
	delay(ARM_MOVING_TIME);
	state=3;
}

void crossBridge(int velocity)
{
	//while (/* Color sensor detection*/)
	//{
		rightMotor.writeMicroseconds(RIGHT_MOTOR_STOP-SPEED_CORRECTION*velocity);
		leftMotor.writeMicroseconds(LEFT_MOTOR_STOP+velocity);
		delay(DELTA_CROSS_TIME);
	//}
	state=4;
}

void uTurn(int velocity)
{
	//while (/* Test alignment*/)
	//{
		rightMotor.writeMicroseconds(RIGHT_MOTOR_STOP+SPEED_CORRECTION*velocity);
		leftMotor.writeMicroseconds(LEFT_MOTOR_STOP+velocity);
		delay(DELTA_ROTATION_TIME);
	//}
        rightMotor.writeMicroseconds(RIGHT_MOTOR_STOP);
	leftMotor.writeMicroseconds(LEFT_MOTOR_STOP);
	state++;
}

void takeBridge()
{
	armMotor.write(ARM_DOWN_POS);
	delay(ARM_MOVING_TIME);
	digitalWrite(MAGNET_PIN, HIGH);
	armMotor.write(ARM_UP_POS);
	delay(ARM_MOVING_TIME);
	digitalWrite(MAGNET_PIN, LOW);
	state=6;
}

void stopCode(int state)
{
	if (state>0)
	{
		// Code is finished
		state=1; // Searching for a new gap
	}
	else
	{
		// error (color sensor LED blink?)
	}
}

int detectGap()
{
	return (analogRead(LEFT_IR_PIN)<TRIGGER_IR_DST || analogRead(RIGHT_IR_PIN)<TRIGGER_IR_DST);
}

void align(int velocity)
{
  int leftIR=analogRead(LEFT_IR_PIN), rightIR=analogRead(RIGHT_IR_PIN);
  while (leftIR>=TRIGGER_IR_DST || rightIR>=TRIGGER_IR_DST)
  {
    if (leftIR>=TRIGGER_IR_DST)
    {
      while(leftIR>=TRIGGER_IR_DST)
      {
        leftMotor.writeMicroseconds(LEFT_MOTOR_STOP+velocity);
        delay(DELTA_ALIGN_TIME);
        leftIR=analogRead(LEFT_IR_PIN);
      }
      leftMotor.writeMicroseconds(LEFT_MOTOR_STOP);
    }
    else if (rightIR>=TRIGGER_IR_DST)
    {
      while(rightIR>=TRIGGER_IR_DST)
      {
        rightMotor.writeMicroseconds(RIGHT_MOTOR_STOP-SPEED_CORRECTION*velocity);
        delay(DELTA_ALIGN_TIME);
        rightIR=analogRead(RIGHT_IR_PIN);
      }
      rightMotor.writeMicroseconds(RIGHT_MOTOR_STOP);
    }
    leftIR=analogRead(LEFT_IR_PIN);
    rightIR=analogRead(RIGHT_IR_PIN);
  }
}

void setup() {
  rightMotor.attach(RIGHT_MOTOR_PIN);
  leftMotor.attach(LEFT_MOTOR_PIN);
  armMotor.attach(ARM_MOTOR_PIN);
  pinMode(LEFT_IR_PIN, INPUT);
  pinMode(RIGHT_IR_PIN, INPUT);
  pinMode(MAGNET_PIN, OUTPUT);
  armMotor.write(ARM_UP_POS);
  delay(ARM_MOVING_TIME);
  state=1;
}

void loop() {
  // activate to keep riding after each gap crossed
  if (state==7)
  {
    state=1;
  }
  switch (state)
  {
    case 0:
    setup();
    break;
    case 1:
    goForwardTillGap(50);
    break;
    case 2:
    putBridge();
    break;
    case 3:
    crossBridge(50);
    break;
    case 4:
    uTurn(50);
    break;
    case 5:
    takeBridge();
    break;
    case 6:
    uTurn(50);
    break;
    default:
    stopCode(state);
  }
}
